@@ -6,7 +6,7 @@ from operator import setitem, getitem
 from types import ModuleType
 from typing import Any, TypeVar, Callable, Dict, Tuple
 
-from testfixtures.resolve import resolve, not_there, Resolved, classmethod_type, class_type
+from testfixtures.resolve import resolve, not_there, Resolved, classmethod_type, class_type, Setter
 from testfixtures.utils import wrap, extend_docstring
 
 import warnings
@@ -30,7 +30,7 @@ class Replacer:
     """
 
     def __init__(self):
-        self.originals: Dict[int, Tuple[Any, Resolved]] = {}
+        self.originals: Dict[Tuple[int, Setter, str], Tuple[Any, Resolved]] = {}
 
     def _replace(self, resolved: Resolved, value):
         if value is not_there:
@@ -47,18 +47,24 @@ class Replacer:
         else:
             resolved.setter(resolved.container, resolved.name, value)
 
-    def __call__(self, target: Any, replacement: R, strict: bool = True,
-                 container: Any = None, accessor: Accessor = None, name: str = None) -> R:
+    def __call__(
+            self,
+            target: Any,
+            replacement: R,
+            strict: bool = True,
+            container: Any = None,
+            accessor: Accessor = None,
+            name: str = None,
+            sep: str = '.',
+    ) -> R:
         """
         Replace the specified target with the supplied replacement.
         """
         if name is None and accessor is not None:
             raise TypeError('accessor is not used unless name is specified')
 
-        if isinstance(target, str):
-            if name is not None:
-                raise TypeError('name cannot be specified when target is a string')
-            resolved = resolve(target, container)
+        if isinstance(target, str) and not name:
+            resolved = resolve(target, container, sep)
         else:
             found = not_there
             if container is None:
@@ -115,7 +121,7 @@ class Replacer:
                 replacement_to_use = staticmethod(replacement)
 
         self._replace(resolved, replacement_to_use)
-        key = id(target)
+        key = resolved.key()
         if key not in self.originals:
             self.originals[key] = target, resolved
         return replacement
@@ -158,9 +164,10 @@ class Replacer:
         name on the class, such as that returned by poorly implemented decorators, then
         ``name`` must be used to provide the correct name.
         """
-        if not callable(attribute):
-            raise TypeError('attribute must be callable')
         name = name or getattr(attribute, '__name__', None)
+        if not callable(attribute):
+            name_text = f' named {name!r} ' if name else ' '
+            raise TypeError(f'attribute{name_text}must be a method')
         container = None
         if isinstance(attribute, classmethod_type):
             for referred in get_referents(attribute):
@@ -216,7 +223,7 @@ class Replacer:
 
 def replace(
         target: Any, replacement: Any, strict: bool = True,
-        container: Any = None, accessor: Accessor = None, name: str = None
+        container: Any = None, accessor: Accessor = None, name: str = None, sep: str ='.'
 ) -> Callable[[Callable], Callable]:
     """
     A decorator to replace a target object for the duration of a test
@@ -224,7 +231,7 @@ def replace(
     """
     r = Replacer()
     return wrap(
-        partial(r.__call__, target, replacement, strict, container, accessor, name),
+        partial(r.__call__, target, replacement, strict, container, accessor, name, sep),
         r.restore
     )
 
@@ -259,14 +266,14 @@ def replace_in_module(target: Any, replacement: Any, module: ModuleType = None):
         yield
 
 
-class Replace(object):
+class Replace:
     """
     A context manager that uses a :class:`Replacer` to replace a single target.
     """
 
     def __init__(
             self, target: Any, replacement: R, strict: bool = True,
-            container: Any = None, accessor: Accessor = None, name: str = None
+            container: Any = None, accessor: Accessor = None, name: str = None, sep: str ='.'
     ):
         self.target = target
         self.replacement = replacement
@@ -274,11 +281,18 @@ class Replace(object):
         self.container: Any = container
         self.accessor: Accessor = accessor
         self.name: str = name
+        self.sep: str = sep
         self._replacer = Replacer()
 
     def __enter__(self) -> R:
         return self._replacer(
-            self.target, self.replacement, self.strict, self.container, self.accessor, self.name
+            self.target,
+            self.replacement,
+            self.strict,
+            self.container,
+            self.accessor,
+            self.name,
+            self.sep,
         )
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -314,12 +328,14 @@ replace_params_doc = """
   :func:`getattr` or :func:`~operator.getitem`.
   
 :param accessor:
-  Either :func:`getattr` or :func:`~operator.getitem`. If not supplied, this will be inferred
-  preferring :func:`~operator.getitem` over :func:`getattr`.
+  Either :func:`getattr` or :func:`~operator.getitem`. If not supplied, this will be inferred.
   
 :param name:
   The name used to access the ``target`` from the ``container`` using the ``accessor``.
   If required but not specified, the ``__name__`` attribute of the ``target`` will be used. 
+  
+:param sep:
+  When ``target`` is a string, this is the separator between traversal segments. 
 """
 
 # add the param docs, so we only have one copy of them!
