@@ -19,7 +19,6 @@ from testfixtures import (
     generator,
     singleton,
     )
-from testfixtures.compat import PY_37_PLUS
 from testfixtures.comparison import compare_sequence, compare_object
 from unittest import TestCase
 
@@ -70,7 +69,7 @@ def check_raises(x_=marker, y_=marker, message=None, regex=None,
         raise AssertionError('No exception raised!')
 
 
-class CompareHelper(object):
+class CompareHelper:
 
     def check_raises(self, *args, **kw):
         check_raises(*args, **kw)
@@ -204,16 +203,10 @@ class TestCompare(CompareHelper, TestCase):
     def test_exception_diff(self):
         e1 = ValueError('some message')
         e2 = ValueError('some other message')
-        if PY_37_PLUS:
-            self.check_raises(
-                e1, e2,
-                "ValueError('some message') != ValueError('some other message')"
-                )
-        else:
-            self.check_raises(
-                e1, e2,
-                "ValueError('some message',) != ValueError('some other message',)"
-                )
+        self.check_raises(
+            e1, e2,
+            "ValueError('some message') != ValueError('some other message')"
+            )
 
     def test_exception_diff_c_wrapper(self):
         e1 = ValueError('some message')
@@ -226,8 +219,9 @@ class TestCompare(CompareHelper, TestCase):
              "'args': ('some message',) (Comparison) "
              "!= ('some other message',) (actual)\n"
              "</C:builtins.ValueError>"
-             " != ValueError('some other message'{message})"
-             ).format(message='' if PY_37_PLUS else ','))
+             " != ValueError('some other message')"
+             )
+        )
 
     def test_sequence_long(self):
         self.check_raises(
@@ -707,9 +701,9 @@ class TestCompare(CompareHelper, TestCase):
 
     def test_sequence_and_generator_strict(self):
         expected = compile(
-            "\(1, 2, 3\) \(<(class|type) 'tuple'>\) \(expected\) != "
-            "<generator object (generator )?at... "
-            "\(<(class|type) 'generator'>\) \(actual\)"
+            r"\(1, 2, 3\) \(<(class|type) 'tuple'>\) \(expected\) != "
+            r"<generator object (generator )?at... "
+            r"\(<(class|type) 'generator'>\) \(actual\)"
             )
         self.check_raises(
             (1, 2, 3), generator(1, 2, 3),
@@ -745,10 +739,10 @@ class TestCompare(CompareHelper, TestCase):
 
     def test_iterable_and_generator_strict(self):
         expected = compile(
-            "range\(1, 4\) \(<(class|type) 'x?range'>\) != "
-            "<generator object (generator )?at... "
-            "\(<(class|type) 'generator'>\)"
-            )
+            r"range\(1, 4\) \(<(class|type) 'x?range'>\) != "
+            r"<generator object (generator )?at... "
+            r"\(<(class|type) 'generator'>\)"
+        )
         self.check_raises(
             range(1, 4), generator(1, 2, 3),
             regex=expected,
@@ -798,7 +792,7 @@ class TestCompare(CompareHelper, TestCase):
         self.check_raises(X, Y, expected)
 
     def test_new_style_classes_same(self):
-        class X(object):
+        class X:
             pass
         compare(X, X)
 
@@ -811,10 +805,10 @@ class TestCompare(CompareHelper, TestCase):
             "test_new_style_classes_different.<locals>.Y'>"
             )
 
-        class X(object):
+        class X:
             pass
 
-        class Y(object):
+        class Y:
             pass
         self.check_raises(X, Y, expected)
 
@@ -928,7 +922,7 @@ b
                     })
 
     def test_extra_comparers_leave_existing(self):
-        class MyObject(object):
+        class MyObject:
             def __init__(self, name):
                 self.name = name
 
@@ -970,7 +964,7 @@ b
             "second:\n[2]"
             )
 
-    def test_strict_okay(self):
+    def test_same_object_strict_okay(self):
         m = object()
         compare(m, m, strict=True)
 
@@ -1466,7 +1460,7 @@ b
     def test_dont_raise(self):
         self.assertEqual(compare('x', 'y', raises=False), "'x' != 'y'")
 
-    class OrmObj(object):
+    class OrmObj:
         def __init__(self, a):
             self.a = a
         def __eq__(self, other):
@@ -1531,7 +1525,7 @@ b
         )
 
     def test_nested_django_orm_in_object(self):
-        class MyObject(object):
+        class MyObject:
             def __init__(self, orm):
                 self.orm = orm
 
@@ -1559,6 +1553,20 @@ b
         m.foo(1, 2, x=3)
         compare(m.mock_calls, m.mock_calls, strict=True)
 
+    def test_mock_call_equal(self):
+        m1 = Mock()
+        m1.foo(1, 2, x=3)
+        m2 = Mock()
+        m2.foo(1, 2, x=3)
+        compare(m1.mock_calls, m2.mock_calls)
+
+    def test_mock_call_equal_strict(self):
+        m1 = Mock()
+        m1.foo(1, 2, x=3)
+        m2 = Mock()
+        m2.foo(1, 2, x=3)
+        compare(m1.mock_calls, m2.mock_calls, strict=True)
+
     def test_calls_different(self):
         m1 = Mock()
         m2 = Mock()
@@ -1596,7 +1604,7 @@ b
         )
 
     def test_calls_args_different_but_same_repr(self):
-        class Annoying(object):
+        class Annoying:
             def __init__(self, x):
                 self.x = x
             def __repr__(self):
@@ -1640,7 +1648,7 @@ b
         )
 
     def test_calls_nested_equal_sub_attributes(self):
-        class Annoying(object):
+        class Annoying:
             def __init__(self, x):
                 self.x = x
             def __repr__(self):
@@ -1726,7 +1734,7 @@ b
 
     def test_inherited_slots(self):
 
-        class Parent(object):
+        class Parent:
             __slots__ = ('a',)
 
         class Child(Parent):
@@ -1749,7 +1757,7 @@ b
 
     def test_empty_child_slots(self):
 
-        class Parent(object):
+        class Parent:
             __slots__ = ('a',)
 
             def __init__(self, a):
@@ -1762,7 +1770,7 @@ b
 
     def test_slots_and_attrs(self):
 
-        class Parent(object):
+        class Parent:
             __slots__ = ('a',)
 
         class Child(Parent):
@@ -1862,7 +1870,7 @@ b
 
     def test_repr_and_attributes_equal(self):
 
-        class Wut(object):
+        class Wut:
             def __repr__(self):
                 return 'Wut'
             def __eq__(self, other):
@@ -1882,7 +1890,7 @@ b
 
     def test_string_with_slotted(self):
 
-        class Slotted(object):
+        class Slotted:
             __slots__ = ['foo']
             def __init__(self, foo):
                 self.foo = foo
@@ -1923,21 +1931,145 @@ b
         expected[2] = expected
         actual = {1: 'foo'}
         actual[2] = actual
-        with ShouldRaise(RecursionError):
-            compare(expected, actual)
+        compare(expected, actual)
 
     def test_self_referential_different(self):
         expected = {1: 'foo'}
         expected[2] = expected
         actual = {1: 'bar'}
         actual[2] = actual
-        with ShouldRaise(RecursionError):
-            compare(expected, actual)
+        self.check_raises(
+            expected,
+            actual,
+            'dict not as expected:\n'
+            '\n'
+            'same:\n'
+            '[2]\n'
+            '\n'
+            'values differ:\n'
+            "1: 'foo' != 'bar'\n"
+            '\n'
+            "While comparing [1]: 'foo' != 'bar'"
+        )
+
+    def test_self_referential_different_but_shows_already_seen(self):
+        ouroboros1 = {}
+        ouroboros1['ouroboros'] = ouroboros1
+        ouroboros2 = {}
+        ouroboros2['ouroboros'] = ouroboros2
+        id2 = str(id(ouroboros2))
+        self.check_raises(
+            {1: ouroboros1, 2: 'foo'},
+            {1: ouroboros2, 2: ouroboros2},
+            'dict not as expected:\n'
+            '\n'
+            'same:\n'
+            '[1]\n'
+            '\n'
+            'values differ:\n'
+            "2: 'foo' != {'ouroboros': <Recursion on dict with id="+id2+">}\n"
+            '\n'
+            "While comparing [2]: 'foo' != "
+            "<AlreadySeen for {'ouroboros': {...}} at [1] with id "+id2+">"
+        )
+
+    def test_self_referential_object_tree(self):
+
+        class Node:
+
+            def __init__(self):
+                self.parent = None
+                self.children = []
+
+            def add(self, child: 'Node'):
+                self.children.append(child)
+                child.parent = self
+
+            def __repr__(self):
+                return f'<Node: {self.children}>'
+
+        expected = Node()
+        expected.add(Node())
+        expected.add(Node())
+
+        actual = Node()
+        actual.add(Node())
+
+        self.check_raises(
+            expected,
+            actual,
+            'Node not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['parent']\n"
+            '\n'
+            'attributes differ:\n'
+            "'children': [<Node: []>, <Node: []>] != [<Node: []>]\n"
+            '\n'
+            'While comparing .children: sequence not as expected:\n'
+            '\n'
+            'same:\n'
+            '[<Node: []>]\n'
+            '\n'
+            'first:\n'
+            '[<Node: []>]\n'
+            '\n'
+            'second:\n'
+            '[]'
+        )
+
+    def test_repeated_object_on_the_left_side_ignore_eq(self):
+        item = [1, 2, 3]
+        compare(expected=[item, item], actual=[[1, 2, 3], [1, 2, 3]], ignore_eq=True)
+
+    def test_repeated_object_on_both_sides_ignore_eq(self):
+        item = [1, 2, 3]
+        compare(expected=[item, item], actual=[item, [1, 2, 3]], ignore_eq=True)
+
+    def test_repeated_object_on_both_sides_left_at_compare_strict_type_same(self):
+        item = [1, 2, 3]
+        compare(expected=[item, item], actual=[item, [1, 2, 3]], strict=True)
+
+    def test_repeated_object_on_both_sides_right_at_compare_strict_type_same(self):
+        item = [1, 2, 3]
+        compare(expected=[item, [1, 2, 3]], actual=[item, item], strict=True)
+
+    def test_repeated_object_on_both_sides_strict_type_different(self):
+        item = [1, 2, 3]
+
+        class MyList(list):
+
+            def __repr__(self):
+                return f'<{type(self).__name__}:{super().__repr__()}>'
+
+        type_repr = repr(MyList)
+
+        self.check_raises(
+            [item, item],
+            [item, MyList((1, 2, 3))],
+            strict=True,
+            message = (
+                'sequence not as expected:\n'
+                '\n'
+                'same:\n'
+                '[[1, 2, 3]]\n'
+                '\n'
+                'first:\n'
+                '[[1, 2, 3]]\n'
+                '\n'
+                'second:\n'
+                '[<MyList:[1, 2, 3]>]\n'
+                '\n'
+                f"While comparing [1]: <AlreadySeen for [1, 2, 3] at ... "
+                f"(<class 'testfixtures.comparison.AlreadySeen'>) != "
+                f"<MyList:[1, 2, 3]> ({type_repr})"
+            )
+        )
 
 
 class TestIgnore(CompareHelper):
 
-    class Parent(object):
+    class Parent:
         def __init__(self, id, other):
             self.id = id
             self.other = other
@@ -1974,9 +2106,9 @@ class TestIgnore(CompareHelper):
         )
 
 
-class TestCompareObject(object):
+class TestCompareObject:
 
-    class Thing(object):
+    class Thing:
         def __init__(self, **kw):
             for k, v in kw.items():
                 setattr(self, k, v)
@@ -2012,7 +2144,7 @@ class MyDerivedClass(BaseClass):
         self.thing = thing
 
 
-class ConcreteBaseClass(object): pass
+class ConcreteBaseClass: pass
 
 
 class ConcreteDerivedClass(ConcreteBaseClass):
