@@ -97,6 +97,18 @@ class Tests(TestCase):
                 call.Popen_instance.wait(),
                 ], Popen.mock.method_calls)
 
+    def test_command_is_bytes(self):
+        Popen = MockPopen()
+        Popen.set_command('a command')
+
+        process = Popen(b'a command')
+
+        compare(process.wait(), 0)
+        compare([
+                call.Popen(b'a command'),
+                call.Popen_instance.wait(),
+                ], Popen.mock.method_calls)
+
     def test_command_is_incorrect_type(self):
         Popen = MockPopen()
         Popen.set_command('a command')
@@ -115,6 +127,18 @@ class Tests(TestCase):
                 call.Popen_instance.wait(),
                 ], Popen.mock.method_calls)
 
+    def test_command_is_sequence_of_bytes(self):
+        Popen = MockPopen()
+        Popen.set_command('a command')
+
+        process = Popen(['a', b'command'])
+
+        compare(process.wait(), 0)
+        compare([
+                call.Popen(['a', b'command']),
+                call.Popen_instance.wait(),
+                ], Popen.mock.method_calls)
+
     def test_command_is_sequence_of_incorrect_type(self):
         Popen = MockPopen()
         Popen.set_command('a command')
