@@ -597,7 +597,7 @@ class TestReplace(TestCase):
     def test_no_name_and_target_string(self):
         x = sample1.X()
         replacer = Replacer()
-        with ShouldRaise(TypeError('name cannot be specified when target is a string')):
+        with ShouldRaise(AttributeError("Original 'aMethod' not found")):
             replacer('.X', lambda cls: 'FOO', name='aMethod', container=sample1)
         assert x.aMethod() is sample1.X
 
@@ -686,6 +686,36 @@ class TestReplace(TestCase):
             from .sample3 import SOME_CONSTANT as sample3_some_constant
             compare(sample3_some_constant, expected=43)
 
+    def test_relative_nested_but_not_present(self):
+        nested = {'b': [1, 2, 3]}
+        with Replacer() as r:
+            with ShouldRaise(AttributeError("Original 'a' not found")):
+                r(container=nested, target='.a.1', replacement=42)
+
+    def test_alternative_separator(self):
+        nested = {'.b': 1, 'c.d': 2, 'e': {'f': 42}}
+
+        with Replacer() as r:
+            r(container=nested, target=':.b', replacement=3, sep=':')
+            r(container=nested, target=':c.d', replacement=4, sep=':')
+            compare(nested, expected={'.b': 3, 'c.d': 4, 'e': {'f': 42}})
+
+        with Replace(container=nested, target=':e:f', replacement=43, sep=':'):
+            compare(nested, expected={'.b': 1, 'c.d': 2, 'e': {'f': 43}})
+
+        @replace(container=nested, target=':e:f', replacement=43, sep=':')
+        def function():
+            compare(nested, expected={'.b': 1, 'c.d': 2, 'e': {'f': 43}})
+
+        function()
+
+    def test_alternative_separator_traversal_not_valid(self):
+        nested = {'.b': 1}
+
+        with Replacer() as r:
+            with ShouldRaise(AttributeError("Original '.a.1' not found")):
+                r(container=nested, target=':.a.1', replacement=3, sep=':')
+
 
 class TestEnviron:
 
@@ -734,6 +764,30 @@ class TestOnClass:
         compare(sample.method(1), expected=2)
         assert SampleClass.__dict__['method'] is original
 
+    def test_attributes_on_class(self):
+
+        class SampleClass:
+            x = 1
+            y = 'a'
+
+        sample = SampleClass()
+
+        with Replacer() as replace:
+            # without names, we get a useful errors:
+            with ShouldRaise(TypeError("attribute named 'x' must be a method")):
+                replace.on_class(SampleClass.x, 2, name='x')
+            with ShouldRaise(TypeError("attribute must be a method")):
+                replace.on_class(SampleClass.y, 'b')
+
+            # okay, so we'll use the full form:
+            replace(SampleClass.x, 2, container=SampleClass, name='x')
+            replace(SampleClass.y, 'b', container=SampleClass, name='y')
+            compare(sample.x, expected=2)
+            compare(sample.y, expected='b')
+
+        compare(sample.x, expected=1)
+        compare(sample.y, expected='a')
+
     def test_method_on_instance(self):
 
         class SampleClass:
@@ -821,19 +875,6 @@ class TestOnClass:
         compare(SampleClass.method(1), expected=2)
         assert SampleClass.__dict__['method'] is original
 
-    def test_not_callable(self):
-
-        class SampleClass:
-
-            FOO = 1
-
-        sample = SampleClass()
-
-        replace = Replacer()
-        with ShouldRaise(TypeError('attribute must be callable')):
-            replace.on_class(SampleClass.FOO, 2)
-        compare(sample.FOO, expected=1)
-
     def test_method_on_class_in_module(self):
         sample = X()
         original = X.__dict__['y']
@@ -903,6 +944,23 @@ class TestOnClass:
         compare(X.bMethod(), expected=2)
         assert X.__dict__['bMethod'] is original
 
+    def test_multiple_methods_on_class(self):
+        original_y = X.y
+        original_a_result = X.aMethod()
+        with Replacer() as replace:
+            # y = replace('testfixtures.tests.sample1.X.y', Mock())
+            # y.return_value = 'mock y'
+            # aMethod = replace('testfixtures.tests.sample1.X.aMethod', Mock())
+            # aMethod.return_value = 'mock method'
+            replace.on_class(X.y, lambda self: 'mock y')
+            replace.on_class(X.aMethod, lambda cls: 'mock method')
+            x = X()
+            compare(x.y(), expected='mock y')
+            compare(x.aMethod(), expected='mock method')
+
+        assert X.y is original_y
+        assert X.aMethod() is original_a_result
+
 
 class TestInModule:
 
